﻿
open NodaTime
open PrayerTracker.Entities

module PgMappings =
    /// Map a row to a Church instance
    let mapToChurch (row: RowReader) =
        { Id               = ChurchId         (row.uuid "id")
          Name             = row.string       "church_name"
          City             = row.string       "city"
          State            = row.string       "state"
          HasVpsInterface  = row.bool         "has_vps_interface"
          InterfaceAddress = row.stringOrNone "interface_address" }

    /// Map a row to a Member instance
    let mapToMember (row: RowReader) =
        { Id           = MemberId         (row.uuid "id")
          SmallGroupId = SmallGroupId     (row.uuid "small_group_id")
          Name         = row.string       "member_name"
          Email        = row.string       "email"
          Format       = row.stringOrNone "email_format" |> Option.map EmailFormat.Parse }

    /// Map a row to a Prayer Request instance
    let mapToPrayerRequest (row: RowReader) =
        { Id             = PrayerRequestId         (row.uuid "id")
          UserId         = UserId                  (row.uuid "user_id")
          SmallGroupId   = SmallGroupId            (row.uuid "small_group_id")
          EnteredDate    = row.fieldValue<Instant> "entered_date"
          UpdatedDate    = row.fieldValue<Instant> "updated_date"
          Requestor      = row.stringOrNone        "requestor"
          Text           = row.string              "request_text"
          NotifyChaplain = row.bool                "notify_chaplain"
          RequestType    = PrayerRequestType.Parse (row.string "request_type")
          Expiration     = Expiration.Parse        (row.string "expiration") }

    /// Map a row to a Small Group instance
    let mapToSmallGroup (row: RowReader) =
        { Id          = SmallGroupId (row.uuid "id")
          ChurchId    = ChurchId     (row.uuid "church_id")
          Name        = row.string   "group_name"
          Preferences =
              { DaysToKeepNew       = row.int      "days_to_keep_new"
                DaysToExpire        = row.int      "days_to_expire"
                LongTermUpdateWeeks = row.int      "long_term_update_weeks"
                EmailFromName       = row.string   "email_from_name"
                EmailFromAddress    = row.string   "email_from_address"
                Fonts               = row.string   "fonts"
                HeadingColor        = row.string   "heading_color"
                LineColor           = row.string   "line_color"
                HeadingFontSize     = row.int      "heading_font_size"
                TextFontSize        = row.int      "text_font_size"
                GroupPassword       = row.string   "group_password"
                IsPublic            = row.bool     "is_public"
                PageSize            = row.int      "page_size"
                TimeZoneId          = TimeZoneId    (row.string "time_zone_id")
                RequestSort         = RequestSort.Parse     (row.string "request_sort")
                DefaultEmailType    = EmailFormat.Parse     (row.string "default_email_type")
                AsOfDateDisplay     = AsOfDateDisplay.Parse (row.string "as_of_date_display") } }

    /// Map a row to a User instance
    let mapToUser (row: RowReader) =
        { Id           = UserId     (row.uuid "id")
          FirstName    = row.string "first_name"
          LastName     = row.string "last_name"
          Email        = row.string "email"
          IsAdmin      = row.bool   "is_admin"
          PasswordHash = row.string "password_hash"
          LastSeen     = row.fieldValueOrNone<Instant> "last_seen"
          SmallGroups  = [] }


open System
open BitBadger.Documents.Sqlite
open Npgsql
open Npgsql.FSharp
open PrayerTracker.Data

task {

    Configuration.useConnectionString (Environment.GetEnvironmentVariable "PT_SQLITE_CONN")
    do! Connection.setUp ()

    let builder = NpgsqlDataSourceBuilder(Environment.GetEnvironmentVariable "PT_PG_CONN")
    let _ = builder.UseNodaTime()
    use source = builder.Build()

    let! churches =
        Sql.fromDataSource source
        |> Sql.query "SELECT * FROM pt.church"
        |> Sql.executeAsync PgMappings.mapToChurch
    for church in churches do
        do! Churches.save church
    printfn "Migrated %d churches" churches.Length

    let! groups =
        Sql.fromDataSource source
        |> Sql.query "SELECT sg.*, lp.* FROM pt.small_group sg
                        INNER JOIN pt.list_preference lp ON lp.small_group_id = sg.id"
        |> Sql.executeAsync PgMappings.mapToSmallGroup
    for group in groups do
        do! SmallGroups.save group
    printfn "Migrated %d groups" groups.Length

    let! members =
        Sql.fromDataSource source
        |> Sql.query "SELECT * from pt.member"
        |> Sql.executeAsync PgMappings.mapToMember
    for mbr in members do
        do! Members.save mbr
    printfn "Migrated %d members" members.Length

    let! requests =
        Sql.fromDataSource source
        |> Sql.query "SELECT * from pt.prayer_request"
        |> Sql.executeAsync PgMappings.mapToPrayerRequest
    for request in requests do
        do! PrayerRequests.save request
    printfn "Migrated %d requests" requests.Length

    let! users =
        Sql.fromDataSource source
        |> Sql.query "SELECT * FROM pt.pt_user"
        |> Sql.executeAsync PgMappings.mapToUser
    for user in users do
        let! groups =
            Sql.fromDataSource source
            |> Sql.query "SELECT small_group_id FROM pt.user_small_group WHERE user_id = @user_id"
            |> Sql.parameters [ "@user_id", Sql.uuid user.Id.Value ]
            |> Sql.executeAsync (fun row -> (row.uuid >> SmallGroupId) "small_group_id")
        do! Users.save { user with SmallGroups = groups }
    printfn "Migrated %d users" users.Length

} |> Async.AwaitTask |> Async.RunSynchronously
